#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>


#define MAX_REQUEST_SIZE (32 * 1024)

typedef struct
{
    int fd;
    char* buf;
    size_t readpos;
    size_t writepos;
} read_info;

void setup_read(read_info* info, int fd)
{
    info->fd = fd;
    info->buf = (char*)malloc(MAX_REQUEST_SIZE);
    info->readpos = 0;
    info->writepos = 0;
}

void close_read(read_info* info)
{
    free(info->buf);
}

bool read_more(read_info* info)
{
    // Request more data to read into the big buffer
    // stop if full
    if (info->writepos == MAX_REQUEST_SIZE)
        return false;
    ssize_t nRead = read(info->fd, &info->buf[info->writepos], MAX_REQUEST_SIZE - info->writepos);
    // negative return values are errors, 0 means EOF
    if (nRead <= 0)
        return false;
    info->writepos += nRead;
    return true;
}

char* read_until_space(read_info* info)
{
    size_t spacePos = info->readpos;
    while (true)
    {
        // read in more data, since we haven't found what we're searching for
        if (spacePos >= info->writepos)
        {
            if (!read_more(info))
                return NULL;
        }
        // found it - stop the loop
        if (info->buf[spacePos] == ' ')
            break;
        ++spacePos;
    }
    
    // finish up the string
    // in c, strings end with \0
    info->buf[spacePos] = '\0';
    // we return a pointer into the big buffer, from where we want to parse until the \0
    char* start = &info->buf[info->readpos];
    // next request starts after that
    info->readpos = (spacePos+1);
    return start;
}


void handle_connection(int connection, char* remoteAddress, uint16_t remotePort)
{
    printf("Incoming connection from %s:%u...\n", remoteAddress, remotePort);
    // initialize helper struct
    read_info info;
    setup_read(&info, connection);

    while (true)
    {
        // get the next string, that is until the next space
        char *next = read_until_space(&info);
        if (!next) break;
        // find the first occurence of ':' (if it exists)
        char *T = strchr(next, ':');
        // strchr returns a NULLptr if it couldn't find the character. Check before dereferencing!!
        if (T)
        {
            // end the string at ':'
            // so our buffer holds two strings
            // +--------+--------+
            // | part 1 | part 2 |
            // +--------+--------+
            *T = 0;
            // dprintf sends something back to the client
            dprintf(connection, "Hello %s, you entered %s\r\n", next, T + 1);
        }
        else
        {
            // no ':', so return error message
            dprintf(connection, "No ':' found, please try again\r\n");
        }
    }
    
    close(connection);
    close_read(&info);
}
