#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <iostream>
#include <string>
#include <stdexcept>
#include <ext/stdio_filebuf.h>
#include <fstream>
#include <array>


class read_info
{
    static constexpr size_t MAX_READ_SIZE = 1024;
    int fd;
    std::string buf;

private:

    void read_more()
    {
        char buff[MAX_READ_SIZE];
        // negative return values are errors, 0 means EOF
        size_t nRead = ::read(fd, buff, MAX_READ_SIZE);
        if (nRead <= 0)
            throw std::runtime_error("Connection lost");
        // add data to internal string
        buf.append(buff, nRead);
    }

public:
    read_info(int connection) : fd(connection) {}
    ~read_info() { ::close(fd); }
    
    std::string read_until_space()
    {
        size_t spacepos = 0;
        while (true)
        {
            // read in more data, since we haven't found what we're searching for
            // maybe have a maximum limit for how much we read?
            if (spacepos >= buf.size())
                read_more();
            // found it - stop the loop
            if (buf[spacepos] == ' ')
                break;
            ++spacepos;
        }
        // extract until space
        auto ret = buf.substr(0, spacepos);
        // remove comsumed data
        buf.erase(0, spacepos + 1);
        return ret;
    }

};


void handle_connection(int connection, char* remoteAddress, uint16_t remotePort)
{
    printf("Incoming connection from %s:%u...\n", remoteAddress, remotePort);
    // initialize helper class
    read_info read(connection);

    try
    {
        while (true)
        {
            // get the next string, that is until the next space
            // throws if connection closed
            std::string next = read.read_until_space();
            // find the first occurence of ':' (if it exists)
            std::size_t T = next.find_first_of(':');
            // find* return npos if not found, otherwise position of char
            if (T != std::string::npos)
            {
                // dprintf sends something back to the client
                dprintf(connection,
                    "Hello %s, you entered %s\r\n",
                    next.substr(0, T).c_str(),
                    next.substr(T + 1).c_str()
                );
            }
            else
            {
                // no ':', so return error message
                dprintf(connection, "No ':' found, please try again\r\n");
            }
        }
    }
    catch (std::exception &ex)
    {
        std::cout << "Encountered error: " << ex.what() << std::endl;
    }
}
